const debug=require('debug')('trip-count-day-route');
const tripsByRouteId=require('./trips-by-route-id');
const serviceAvailability=require('./service-availability');

//remote time from a Date object
function removeTime(date = new Date()) {
    return new Date(
	date.getFullYear(),
	date.getMonth(),
	date.getDate()
    );
};

/*
 * Compute number of trips belonging to a route provided that day
 * @param routeId id for the route to be considered
 * @param day the day to be considered
 * @return number of trips
 */
async function get(routeId = 0, day = 0) {
    //debug('trip-count-day-route start...');
    //debug('trip-count-day-route routeId: '+routeId);
    //debug('trip-count-day-route day ts: '+day);

    //convert day into date without timestamp
    const dayMs = day * 1000;
    const dayDate = new Date(dayMs);
    const dayDateNoTime = removeTime(dayDate);
    const dayDateNoTimeTs = dayDateNoTime.getTime();
    //debug('trip-count-day-route day as dateNoTimeTs: '+dayDateNoTimeTs);

    //get trips array
    const aryTrips=await tripsByRouteId.get(routeId);
    //debug('trip-count-day-route aryTrips.length: '+aryTrips.length);
    //debug('trip-count-day-route aryTrips: '+JSON.stringify(aryTrips));

    //init services map
    const mapServices=new Map();
    //debug('trip-count-day-route mapServices.size: '+mapServices.size);

    //compute trip count
    let tripCount=0;
    //iterate over trips
    for(var i=0;i<aryTrips.length;i++){
	//debug('trip-count-day-route i: '+i);
	const tripId=aryTrips[i].trip_id;
	//debug('trip-count-day-route tripId: '+tripId);
	const serviceId=aryTrips[i].service_id;
	//debug('trip-count-day-route serviceId: '+serviceId);

	//get service
	let service=[];
	if(!mapServices.has(serviceId)){
	    service=await serviceAvailability.get(serviceId);
	    mapServices.set(serviceId,service);
	}else{
	    service=mapServices.get(serviceId);
	}
	//debug('trip-count-day-route service.length: '+service.length);

	//iterate over service availabiltiy
	for(var j=0;j<service.length;j++){
	    //debug('trip-count-day-route service j: '+j);
	    //debug('trip-count-day-route service[j]: '+service[j]);
	    //debug('trip-count-day-route dayDateNoTimeTs: '+dayDateNoTimeTs);
	    if(service[j]===dayDateNoTimeTs){
		tripCount++;
		//debug('trip-count-day-route tripCount: '+tripCount);
	    }
	}
    }
    //debug('trip-count-day-route tripCount: '+tripCount);
    //debug('trip-count-day-route done.');
    return tripCount;
};      
module.exports = {
    get
}
